@@ -45,6 +45,12 @@ class Stream(object):
     def write(self, data):
         self.f.write(data)
 
+    def peek(self, n):
+        pos = self.tell()
+        ret = self.read(n)
+        self.seek(pos)
+        return ret
+
     def writeLine(self, data):
         self.f.write(data + "\n")
 
@@ -112,21 +118,57 @@ class Stream(object):
     def writeInt(self, num):
         self.f.write(struct.pack(self.endian + "i", num))
 
+    def writeIntAt(self, pos, num):
+        current = self.tell()
+        self.seek(pos)
+        self.writeInt(num)
+        self.seek(current)
+
     def writeUInt(self, num):
         self.f.write(struct.pack(self.endian + "I", num))
 
+    def writeUIntAt(self, pos, num):
+        current = self.tell()
+        self.seek(pos)
+        self.writeUInt(num)
+        self.seek(current)
+
     def writeShort(self, num):
         self.f.write(struct.pack(self.endian + "h", num))
 
+    def writeShortAt(self, pos, num):
+        current = self.tell()
+        self.seek(pos)
+        self.writeShort(num)
+        self.seek(current)
+
     def writeUShort(self, num):
         self.f.write(struct.pack(self.endian + "H", num))
 
+    def writeUShortAt(self, pos, num):
+        current = self.tell()
+        self.seek(pos)
+        self.writeUShort(num)
+        self.seek(current)
+
     def writeByte(self, num):
         self.f.write(struct.pack("B", num))
 
+    def writeByteAt(self, pos, num):
+        current = self.tell()
+        self.seek(pos)
+        self.writeByte(num)
+        self.seek(current)
+
     def writeSByte(self, num):
         self.f.write(struct.pack("b", num))
 
+    def writeSByteAt(self, pos, num):
+        current = self.tell()
+        self.seek(pos)
+        self.writeSByte(num)
+        self.seek(current)
+
     def writeHalf(self, num):
         if self.half is None:
             self.half = num
@@ -138,8 +180,9 @@ class Stream(object):
         self.f.write(str.encode("ascii"))
 
     def writeZero(self, num):
-        for i in range(num):
-            self.writeByte(0)
+        if num > 0:
+            for i in range(num):
+                self.writeByte(0)
 
 
 # Logging
@@ -337,7 +380,7 @@ def readEncodedString(f, encoding="shift_jis"):
     return sjis
 
 
-def detectEncodedString(f, encoding="shift_jis"):
+def detectEncodedString(f, encoding="shift_jis", startascii=[0x25]):
     ret = ""
     sjis = 0
     while True:
@@ -346,7 +389,7 @@ def detectEncodedString(f, encoding="shift_jis"):
             ret += "|"
         elif b1 == 0x00:
             break
-        elif b1 >= 28 and b1 <= 126 and (len(ret) > 0 or b1 == 0x25):
+        elif b1 >= 28 and b1 <= 126 and (len(ret) > 0 or b1 in startascii):
             ret += chr(b1)
         else:
             b2 = f.readByte()
